<?php

namespace BooklyPro\Frontend\Modules\SearchForm;

use Bookly\Lib as BooklyLib;
use BooklyPro\Lib;

/**
 * Class Ajax
 *
 * @package BooklyPro\Frontend\Modules\SearchForm
 */
class Ajax extends BooklyLib\Base\Ajax
{
    public static function searchFormGetServices()
    {
        $list = array();
        $filters = self::parameter( 'filters' );
        $date = self::parameter( 'date' );
        if ( ! isset( $filters['locations'] ) ) {
            $filters['locations'] = array( 0 );
        }
        foreach ( $filters['services'] as $service_id ) {
            $service = BooklyLib\Entities\Service::find( $service_id );
            foreach ( $filters['staff'] as $staff_id ) {
                foreach ( $filters['locations'] as $location_id ) {
                    $chain_item = new BooklyLib\ChainItem();
                    $chain_item
                        ->setStaffIds( array( $staff_id ) )
                        ->setServiceId( $service_id )
                        ->setNumberOfPersons( $service->getCapacityMin() )
                        ->setQuantity( 1 )
                        ->setLocationId( $location_id )
                        ->setUnits( $service->getUnitsMin() )
                        ->setExtras( array() );

                    $chain = new BooklyLib\Chain();
                    $chain->add( $chain_item );

                    $scheduler = new BooklyLib\Scheduler( $chain, date_create( $date )->format( 'Y-m-d 00:00' ), date_create( $date )->format( 'Y-m-d' ), 'daily', array( 'every' => 1 ), array(), false );
                    $schedule = $scheduler->scheduleForFrontend( 1 );
                    if ( isset( $schedule[0]['options'] ) && count( $schedule[0]['options'] ) ) {
                        $list[] = array(
                            'service_id' => $service_id,
                            'staff_id' => $staff_id,
                            'location_id' => $location_id,
                        );
                    }
                }
            }
        }
        wp_send_json_success( $list );
    }

    public static function searchFormGetSlots()
    {
        $date = self::parameter( 'date' );
        $service_id = self::parameter( 'service_id' );
        $staff_id = self::parameter( 'staff_id' );
        $location_id = self::parameter( 'location_id' );
        $nop = self::parameter( 'nop' );
        $units = self::parameter( 'units', 1 );

        $chain_item = new BooklyLib\ChainItem();
        $chain_item
            ->setStaffIds( array( $staff_id ) )
            ->setServiceId( $service_id )
            ->setNumberOfPersons( $nop )
            ->setQuantity( 1 )
            ->setLocationId( BooklyLib\Proxy\Locations::prepareStaffLocationId( $location_id, $staff_id ) )
            ->setUnits( $units )
            ->setExtras( array() );

        $chain = new BooklyLib\Chain();
        $chain->add( $chain_item );

        $scheduler = new BooklyLib\Scheduler( $chain, date_create( $date )->format( 'Y-m-d 00:00' ), date_create( $date )->format( 'Y-m-d' ), 'daily', array( 'every' => 1 ), array(), false );
        $schedule = $scheduler->scheduleForFrontend( 1 );

        if ( isset( $schedule[0]['options'] ) ) {
            foreach ( $schedule[0]['options'] as &$option ) {
                $staff_service = new BooklyLib\Entities\StaffService();
                $location_id = BooklyLib\Proxy\Locations::prepareStaffLocationId( $location_id, $staff_id ) ?: null;
                $staff_service->loadBy( compact( 'staff_id', 'service_id', 'location_id' ) );
                $slot = json_decode( $option['value'] );
                $date = strtotime( $slot[0][2] );
                $time = substr( $slot[0][2], 11 );
                $option['price'] = BooklyLib\Utils\Price::format( BooklyLib\Proxy\SpecialHours::adjustPrice( $staff_service->getPrice(), $staff_id, $service_id, $location_id, $time, $units, date( 'w', $date ) + 1 ) );
                $option['datetime'] = BooklyLib\Utils\DateTime::formatDateTime( $date );
            }
        }

        wp_send_json_success( $schedule );
    }

    public static function searchFormSaveAppointment()
    {
        $customer = self::parameter( 'customer' );

        $errors = array();

        if ( $customer['email'] === '' || ! is_email( $customer['email'] ) ) {
            $errors[] = 'email';
        }

        foreach ( array( 'phone', 'first_name', 'last_name' ) as $field ) {
            if ( $customer[ $field ] === '' ) {
                $errors[] = $field;
            }
        }

        if ( ! empty( $errors ) ) {
            wp_send_json_error( $errors );
        }


        $service_id = self::parameter( 'service_id' );
        $staff_id = self::parameter( 'staff_id' );
        $location_id = self::parameter( 'location_id' );
        $nop = self::parameter( 'nop' );
        $units = self::parameter( 'units', 1 );
        $slot = self::parameter( 'slot' );
        $slots = json_decode( $slot['value'], true );

        $service = BooklyLib\Entities\Service::find( $service_id );

        // Check & save appointment
        $userData = new BooklyLib\UserBookingData( null );
        $userData
            ->setFirstName( $customer['first_name'] )
            ->setLastName( $customer['last_name'] )
            ->setEmail( $customer['email'] )
            ->setPhone( $customer['phone'] )
            ->setSlots( $slots );

        $cart_item = new BooklyLib\CartItem();
        $cart_item
            ->setStaffIds( array( $staff_id ) )
            ->setServiceId( $service_id )
            ->setNumberOfPersons( $nop )
            ->setLocationId( BooklyLib\Proxy\Locations::prepareStaffLocationId( $location_id, $staff_id ) )
            ->setSlots( $slots )
            ->setUnits( $units )
            ->setExtras( array() );
        $userData->cart->add( $cart_item );

        if ( $userData->cart->getFailedKey() !== null ) {
            $errors[] = 'slots';
            wp_send_json_error( $errors );
        }

        $cart_info = $userData->cart->getInfo();

        $payment = new BooklyLib\Entities\Payment();
        $payment
            ->setType( BooklyLib\Entities\Payment::TYPE_LOCAL )
            ->setStatus( BooklyLib\Entities\Payment::STATUS_PENDING )
            ->setPaidType( BooklyLib\Entities\Payment::PAY_IN_FULL )
            ->setTotal( $cart_info->getTotal() )
            ->setTax( $cart_info->getTotalTax() )
            ->setPaid( 0 )
            ->save();

        $order = $userData->save( $payment );
        $order_item = current( $order->getItems() );
        $appointment = $order_item->getAppointment();

        $payment->setDetailsFromOrder( $order, $cart_info )->save();
        // Send notifications.
        BooklyLib\Notifications\Cart\Sender::send( $order );

        $staff = BooklyLib\Entities\Staff::find( $staff_id );

        $ics = new BooklyLib\Utils\Ics\Feed();
        $ics->addEvent( $appointment->getStartDate(), $appointment->getEndDate(), $service->getTranslatedTitle(), $staff->getFullName(), $location_id );
        wp_send_json_success( array(
            'ics' => add_query_arg(
                array(
                    'cht' => 'qr',
                    'chs' => '298x298',
                    'chl' => urlencode( $ics->render() ),
                ),
                'https://chart.googleapis.com/chart'
            ),
        ) );
    }

    /**
     * @inheritDoc
     */
    protected static function permissions()
    {
        return array( '_default' => 'anonymous' );
    }
}